@@ -16,10 +16,9 @@ use std::collections::{HashMap, HashSet};
 use std::env;
 use std::io;
 use std::path::PathBuf;
-use structopt::StructOpt;
 
 /// High level API for building wheels from a crate which is also used for the CLI
-#[derive(Debug, Serialize, Deserialize, StructOpt, Clone, Eq, PartialEq)]
+#[derive(Debug, Serialize, Deserialize, clap::Parser, Clone, Eq, PartialEq)]
 #[serde(default)]
 pub struct BuildOptions {
     /// Control the platform tag on linux.
@@ -34,38 +33,39 @@ pub struct BuildOptions {
     /// The default is the lowest compatible `manylinux` tag, or plain `linux` if nothing matched
     ///
     /// This option is ignored on all non-linux platforms
-    #[structopt(
+    #[clap(
         name = "compatibility",
         long = "compatibility",
         alias = "manylinux",
         parse(try_from_str)
     )]
     pub platform_tag: Option<PlatformTag>,
-    #[structopt(short, long)]
+    #[clap(short, long)]
     /// The python versions to build wheels for, given as the names of the
     /// interpreters. Uses autodiscovery if not explicitly set.
     pub interpreter: Option<Vec<PathBuf>>,
     /// Which kind of bindings to use. Possible values are pyo3, rust-cpython, cffi and bin
-    #[structopt(short, long)]
+    #[clap(short, long)]
     pub bindings: Option<String>,
-    #[structopt(
-        short = "m",
-        long = "manifest-path",
-        parse(from_os_str),
-        default_value = "Cargo.toml",
-        name = "PATH"
-    )]
+    #[clap(short = 'm', long = "manifest-path", parse(from_os_str), name = "PATH")]
     /// The path to the Cargo.toml
-    pub manifest_path: PathBuf,
+    pub manifest_path: Option<PathBuf>,
     /// The directory to store the built wheels in. Defaults to a new "wheels"
     /// directory in the project's target directory
-    #[structopt(short, long, parse(from_os_str))]
+    #[clap(short, long, parse(from_os_str))]
     pub out: Option<PathBuf>,
     /// Don't check for manylinux compliance
-    #[structopt(long = "skip-auditwheel")]
+    #[clap(long = "skip-auditwheel")]
     pub skip_auditwheel: bool,
+    /// For manylinux targets, use zig to ensure compliance for the chosen manylinux version
+    ///
+    /// Default to manylinux2010/manylinux_2_12 if you do not specify an `--compatibility`
+    ///
+    /// Make sure you installed zig with `pip install maturin[zig]`
+    #[clap(long)]
+    pub zig: bool,
     /// The --target option for cargo
-    #[structopt(long, name = "TRIPLE", env = "CARGO_BUILD_TARGET")]
+    #[clap(long, name = "TRIPLE", env = "CARGO_BUILD_TARGET")]
     pub target: Option<String>,
     /// Extra arguments that will be passed to cargo as `cargo rustc [...] [arg1] [arg2] -- [...]`
     ///
@@ -74,16 +74,16 @@ pub struct BuildOptions {
     /// Note that maturin invokes cargo twice: Once as `cargo metadata` and then as `cargo rustc`.
     /// maturin tries to pass only the shared subset of options to cargo metadata, but this is may
     /// be a bit flaky.
-    #[structopt(long = "cargo-extra-args")]
+    #[clap(long = "cargo-extra-args")]
     pub cargo_extra_args: Vec<String>,
     /// Extra arguments that will be passed to rustc as `cargo rustc [...] -- [...] [arg1] [arg2]`
     ///
     /// Use as `--rustc-extra-args="--my-arg"`
-    #[structopt(long = "rustc-extra-args")]
+    #[clap(long = "rustc-extra-args")]
     pub rustc_extra_args: Vec<String>,
     /// Control whether to build universal2 wheel for macOS or not.
     /// Only applies to macOS targets, do nothing otherwise.
-    #[structopt(long)]
+    #[clap(long)]
     pub universal2: bool,
 }
 
@@ -93,9 +93,10 @@ impl Default for BuildOptions {
             platform_tag: None,
             interpreter: Some(vec![]),
             bindings: None,
-            manifest_path: PathBuf::from("Cargo.toml"),
+            manifest_path: None,
             out: None,
             skip_auditwheel: false,
+            zig: false,
             target: None,
             cargo_extra_args: Vec::new(),
             rustc_extra_args: Vec::new(),
@@ -105,6 +106,41 @@ impl Default for BuildOptions {
 }
 
 impl BuildOptions {
+    /// Get cargo manifest file path
+    fn manifest_path(&self) -> Result<PathBuf> {
+        // use command line argument if specified
+        if let Some(path) = &self.manifest_path {
+            return Ok(path.clone());
+        }
+        // check `manifest-path` option in pyproject.toml
+        let current_dir = env::current_dir()
+            .context("Failed to detect current directory ಠ_ಠ")?
+            .canonicalize()?;
+        let pyproject = PyProjectToml::new(&current_dir).context("pyproject.toml is invalid")?;
+        if let Some(path) = pyproject.manifest_path() {
+            println!("🔗 Found cargo manifest path in pyproject.toml");
+            // pyproject.toml must be placed at top directory
+            let manifest_dir = path
+                .parent()
+                .context("missing parent directory")?
+                .canonicalize()?;
+            if !manifest_dir.starts_with(&current_dir) {
+                bail!("Cargo.toml can not be placed outside of the directory containing pyproject.toml");
+            }
+            return Ok(path.to_path_buf());
+        }
+        // check Cargo.toml in current directory
+        let path = PathBuf::from("Cargo.toml");
+        if path.exists() {
+            Ok(path)
+        } else {
+            Err(format_err!(
+                "Can't find {} (in {})",
+                path.display(),
+                current_dir.display()
+            ))
+        }
+    }
     /// Tries to fill the missing metadata for a BuildContext by querying cargo and python
     pub fn into_build_context(
         self,
@@ -112,21 +148,10 @@ impl BuildOptions {
         strip: bool,
         editable: bool,
     ) -> Result<BuildContext> {
-        let manifest_file = &self.manifest_path;
-        if !manifest_file.exists() {
-            let current_dir =
-                env::current_dir().context("Failed to detect current directory ಠ_ಠ")?;
-            bail!(
-                "Can't find {} (in {})",
-                self.manifest_path.display(),
-                current_dir.display()
-            );
-        }
-
+        let manifest_file = self.manifest_path()?;
         if !manifest_file.is_file() {
             bail!(
-                "{} (resolved to {}) is not the path to a Cargo.toml",
-                self.manifest_path.display(),
+                "{} is not the path to a Cargo.toml",
                 manifest_file.display()
             );
         }
@@ -188,7 +213,7 @@ impl BuildOptions {
         let cargo_metadata_extra_args = extract_cargo_metadata_args(&cargo_extra_args)?;
 
         let result = MetadataCommand::new()
-            .manifest_path(&self.manifest_path)
+            .manifest_path(&manifest_file)
             .other_options(cargo_metadata_extra_args)
             .exec();
 
@@ -271,14 +296,28 @@ impl BuildOptions {
         let strip = pyproject.map(|x| x.strip()).unwrap_or_default() || strip;
         let skip_auditwheel =
             pyproject.map(|x| x.skip_auditwheel()).unwrap_or_default() || self.skip_auditwheel;
-        let platform_tag = self.platform_tag.or_else(|| {
-            pyproject.and_then(|x| {
-                if x.compatibility().is_some() {
-                    args_from_pyproject.push("compatibility");
-                }
-                x.compatibility()
+        let platform_tag = self
+            .platform_tag
+            .or_else(|| {
+                pyproject.and_then(|x| {
+                    if x.compatibility().is_some() {
+                        args_from_pyproject.push("compatibility");
+                    }
+                    x.compatibility()
+                })
             })
-        });
+            .or(if self.zig {
+                if target.is_musl_target() {
+                    // Zig bundles musl 1.2
+                    Some(PlatformTag::Musllinux { x: 1, y: 2 })
+                } else {
+                    // With zig we can compile to any glibc version that we want, so we pick the lowest
+                    // one supported by the rust compiler
+                    Some(target.get_minimum_manylinux_tag())
+                }
+            } else {
+                None
+            });
         if platform_tag == Some(PlatformTag::manylinux1()) {
             eprintln!("⚠️  Warning: manylinux1 is unsupported by the Rust compiler.");
         }
@@ -297,11 +336,12 @@ impl BuildOptions {
             metadata21,
             crate_name: crate_name.to_string(),
             module_name,
-            manifest_path: self.manifest_path,
+            manifest_path: manifest_file,
             out: wheel_dir,
             release,
             strip,
             skip_auditwheel,
+            zig: self.zig,
             platform_tag,
             cargo_extra_args,
             rustc_extra_args,
